@@ -1,6 +1,6 @@
 package Spreadsheet::ParseXLSX;
 our $AUTHORITY = 'cpan:DOY';
-$Spreadsheet::ParseXLSX::VERSION = '0.26';
+$Spreadsheet::ParseXLSX::VERSION = '0.27';
 use strict;
 use warnings;
 use 5.010;
@@ -188,6 +188,7 @@ sub _parse_sheet {
     my $default_column_width = 10;
 
     my %cells;
+    my $row_idx = 0;
 
     my $sheet_xml = $self->_new_twig(
         twig_roots => {
@@ -295,15 +296,6 @@ sub _parse_sheet {
                 $twig->purge;
             },
 
-            's:row' => sub {
-                my ( $twig, $row ) = @_;
-
-                $row_heights[ $row->att('r') - 1 ] = $row->att('ht');
-                $rows_hidden[ $row->att('r') - 1 ] = $self->_xml_boolean($row->att('hidden'));
-
-                $twig->purge;
-            },
-
             's:selection' => sub {
                 my ( $twig, $selection ) = @_;
 
@@ -331,9 +323,26 @@ sub _parse_sheet {
 
             's:sheetData/s:row' => sub {
                 my ( $twig, $row_elt ) = @_;
+                my $explicit_row_idx = $row_elt->att('r');
+                $row_idx = $explicit_row_idx - 1 if defined $explicit_row_idx;
 
+                $row_heights[$row_idx] = $row_elt->att('ht');
+                $rows_hidden[$row_idx] = $self->_xml_boolean($row_elt->att('hidden'));
+
+                my $col_idx = 0;
                 for my $cell ( $row_elt->children('s:c') ){
-                    my ($row, $col) = $self->_cell_to_row_col($cell->att('r'));
+                    my $loc = $cell->att('r');
+                    my ($row, $col);
+                    if ($loc) {
+                        ($row, $col) = $self->_cell_to_row_col($loc);
+                        if ($row != $row_idx) {
+                            warn "mismatched coords: got $loc for cell in row $row_idx";
+                        }
+                        $col_idx = $col - 1;
+                    }
+                    else {
+                        ($row, $col) = ($row_idx, $col_idx);
+                    }
                     $sheet->{MaxRow} = $row
                         if $sheet->{MaxRow} < $row;
                     $sheet->{MaxCol} = $col
@@ -408,9 +417,11 @@ sub _parse_sheet {
                     );
                     $cells{"$row;$col"} = $cell;
                     $sheet->{Cells}[$row][$col] = $cell;
+                    $col_idx++;
                 }
 
                 $twig->purge;
+                $row_idx++;
             },
         }
     );
@@ -1103,7 +1114,7 @@ Spreadsheet::ParseXLSX - parse XLSX files
 
 =head1 VERSION
 
-version 0.26
+version 0.27
 
 =head1 SYNOPSIS
 
